require 'csv'

require_relative 'game_turn'
require_relative 'player'
require_relative 'treasure_trove'

# Games have a title, add players to the game, and play rounds of the game
class Game
  attr_reader :title

  def initialize(title = 'default')
    @title = title
    @players = []
  end

  # Adds a player to the game
  #
  # @param player [Player] player to add to the game
  def add_player(player)
    @players.push player
  end

  # Plays the game
  #
  # @param rounds [Integer] number of turns each player gets
  def play(rounds = 1)
    puts "There are #{@players.length} players in #{title}:"
    print_treasures
    1.upto rounds do |round|
      play_round round
    end
  end

  # Prints all the available treasures and their points
  def print_treasures
    treasures = TreasureTrove::TREASURES
    puts "There are #{treasures.length} treasures to be found:"
    treasures.each do |treasure|
      puts "A #{treasure.name} is worth #{treasure.points} points"
    end
  end

  # Plays a single round of the game for all players
  def play_round(round)
    puts "\nRound #{round}"
    @players.each do |player|
      GameTurn.take_turn player
      puts player
    end
  end

  # :reek:TooManyStatements
  def print_stats
    strong, wimpy = @players.partition(&:strong?)
    puts "\n#{title} Statistics:"
    print_players strong
    print_players wimpy, 'wimpy'
    print_points
    print_high_scores
  end

  def print_players(players, label = 'strong')
    puts "#{players.length} #{label} players:"
    players.each { |player| puts "#{player.name} (#{player.health})" }
  end

  def print_points
    @players.sort.each do |player|
      puts "\n#{player.name}'s point totals:"
      print_player_treasures player
    end
  end

  def print_player_treasures(player)
    player.each_found_treasure do |treasure|
      puts "#{treasure.points} total #{treasure.name} points"
    end
    puts "#{player.points} grand total points"
  end

  def load_players(player_file)
    CSV.read(player_file).each do |player, health|
      add_player Player.new player, health.to_i
    end
  end

  def print_high_scores(out = $stdout)
    out.puts "\n#{title} High Scores:"
    @players.sort.each do |player|
      out.puts "#{player.name.ljust 60, '.'} #{player.score}"
    end
  end

  def save_high_scores(to_file = 'high_scores.txt')
    File.open(to_file, 'w') do |file|
      print_high_scores file
    end
  end
end
